@@ -1,16 +1,20 @@
 #!/usr/bin/env python
 # encoding: utf-8
 
-import random
+from __future__ import print_function
+
+import argparse
+import math
 import os
 import os.path
-import argparse
+import random
 import re
-import math
 import sys
 
+from io import open
+
 __LICENSE__ = """
-Copyright (c) 2011 - 2017, Steven Tobin and Contributors.
+Copyright (c) 2011 - 2018, Steven Tobin and Contributors.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -112,25 +116,25 @@ def generate_wordlist(wordfile=None,
     Generate a word list from either a kwarg wordfile, or a system default
     valid_chars is a regular expression match condition (default - all chars)
     """
-    
+
     # deal with inconsistent min and max, erring toward security
     if min_length > max_length:
         max_length = min_length
     wordfile = locate_wordfile(wordfile)
 
-    words = []
+    words = set()
 
     regexp = re.compile("^{0}{{{1},{2}}}$".format(valid_chars,
                                                   min_length,
                                                   max_length))
     # read words from file into wordlist
-    with open(wordfile) as wlf:
+    with open(wordfile, encoding='utf-8') as wlf:
         for line in wlf:
             thisword = line.strip()
             if regexp.match(thisword) is not None:
-                words.append(thisword)
+                words.add(thisword)
 
-    return list(set(words))  # deduplicate, just in case
+    return list(words)  # deduplicate, just in case
 
 
 def wordlist_to_worddict(wordlist):
@@ -220,11 +224,89 @@ def try_input(prompt, validate):
     return validate(answer)
 
 
+def alternating_case(words):
+    """
+    Set EVERY OTHER word to UPPER case.
+    """
+    return [word.upper()
+            if i % 2 == 0
+            else word
+            for i, word in enumerate(lower_case(words))]
+
+
+def upper_case(words):
+    """
+    Set ALL words to UPPER case.
+    """
+    return [w.upper() for w in words]
+
+def first_upper_case(words):
+    """
+    Set First character of each word to UPPER case.
+    """
+    return [w.capitalize() for w in words]
+
+def lower_case(words):
+    """
+    Set ALL words to LOWER case.
+    """
+    return [w.lower() for w in words]
+
+
+def random_case(words, testing=False):
+    """
+    Set RANDOM words to UPPER case.
+    """
+    def make_upper(word):
+        """Return 'word.upper()' on a random basis."""
+        if testing:
+            random.seed(word)
+
+        if random.choice([True, False]):
+            return word.upper()
+        else:
+            return word
+
+    return [make_upper(word) for word in lower_case(words)]
+
+
+CASE_METHODS = {"alternating": alternating_case,
+                "upper": upper_case,
+                "lower": lower_case,
+                "random": random_case,
+                "first": first_upper_case,
+                }
+
+
+def set_case(words, method="lower", testing=False):
+    """
+    Perform capitalization on some or all of the strings in `words`.
+
+    Default method is "lower".
+
+    Args:
+        words (list):   word list generated by `choose_words()` or
+                        `find_acrostic()`.
+        method (str):   one of {"alternating", "upper", "lower",
+                        "random"}.
+        testing (bool): only affects method="random".
+                        If True: the random seed will be set to each word
+                        prior to choosing True or False before setting the
+                        case to upper. This way we can test that random is
+                        working by giving different word lists.
+    """
+    if (method == "random") and (testing):
+        return random_case(words, testing=True)
+    else:
+        return CASE_METHODS[method](words)
+
+
 def generate_xkcdpassword(wordlist,
                           numwords=6,
                           interactive=False,
                           acrostic=False,
-                          delimiter=" "):
+                          delimiter=" ",
+                          case="lower"):
     """
     Generate an XKCD-style password from the words in wordlist.
     """
@@ -235,14 +317,17 @@ def generate_xkcdpassword(wordlist,
     if acrostic:
         worddict = wordlist_to_worddict(wordlist)
 
+    def gen_passwd():
+        if not acrostic:
+            words = choose_words(wordlist, numwords)
+        else:
+            words = find_acrostic(acrostic, worddict)
+
+        return delimiter.join(set_case(words, method=case))
+
     # useful if driving the logic from other code
     if not interactive:
-        if not acrostic:
-            passwd = delimiter.join(choose_words(wordlist, numwords))
-        else:
-            passwd = delimiter.join(find_acrostic(acrostic, worddict))
-
-        return passwd
+        return gen_passwd()
 
     # else, interactive session
     # define input validators
@@ -277,10 +362,7 @@ def generate_xkcdpassword(wordlist,
     accepted = False
 
     while not accepted:
-        if not acrostic:
-            passwd = delimiter.join(choose_words(wordlist, numwords))
-        else:
-            passwd = delimiter.join(find_acrostic(acrostic, worddict))
+        passwd = gen_passwd()
         print("Generated: " + passwd)
         accepted = try_input("Accept? [yN] ", accepted_validator)
 
@@ -291,12 +373,16 @@ def emit_passwords(wordlist, options):
     """ Generate the specified number of passwords and output them. """
     count = options.count
     while count > 0:
-        print(generate_xkcdpassword(
-            wordlist,
-            interactive=options.interactive,
-            numwords=options.numwords,
-            acrostic=options.acrostic,
-            delimiter=options.delimiter))
+        print(
+            generate_xkcdpassword(
+                wordlist,
+                interactive=options.interactive,
+                numwords=options.numwords,
+                acrostic=options.acrostic,
+                delimiter=options.delimiter,
+                case=options.case,
+            ),
+            end=options.separator)
         count -= 1
 
 
@@ -317,7 +403,8 @@ class XkcdPassArgumentParser(argparse.ArgumentParser):
                 "Specify that the file WORDFILE contains the list"
                 " of valid words from which to generate passphrases."
                 " Provided wordfiles: eff-long (default), eff-short,"
-                " eff-special, legacy, spa-mich (Spanish), fin-kotus (Finnish)"))
+                " eff-special, legacy, spa-mich (Spanish), fin-kotus (Finnish)"
+                " ita-wiki (Italian), ger-anlx (German)"))
         self.add_argument(
             "--min",
             dest="min_length", type=int, default=5, metavar="MIN_LENGTH",
@@ -358,6 +445,20 @@ class XkcdPassArgumentParser(argparse.ArgumentParser):
             "-d", "--delimiter",
             dest="delimiter", default=" ", metavar="DELIM",
             help="Separate words within a passphrase with DELIM.")
+        self.add_argument(
+            "-s", "--separator",
+            dest="separator", default="\n", metavar="SEP",
+            help="Separate generated passphrases with SEP.")
+        self.add_argument(
+            "-C", "--case",
+            dest="case", type=str, metavar="CASE",
+            choices=list(CASE_METHODS.keys()), default="lower",
+            help=(
+                "Choose the method for setting the case of each word "
+                "in the passphrase. "
+                "Choices: {cap_meths} (default: 'lower').".format(
+                    cap_meths=list(CASE_METHODS.keys())
+                )))
         self.add_argument(
             "--allow-weak-rng",
             action="store_true", dest="allow_weak_rng", default=False,
